@@ -4,7 +4,7 @@ import os
 import json
 from pathlib import Path
 from anc.cli.util import click_group, console
-from .util import ConfigManager
+from .util import get_custom_code_base_from_env
 import uuid
 import datetime
 from rich.panel import Panel
@@ -12,11 +12,11 @@ from rich.table import Table
 from rich.text import Text
 from rich.console import Console
 from rich import box
-from .operators.eval_operator import trigger_eval_job, display_evaluation_status, trigger_eval_sweep, eval_log_print
+from .operators.eval_operator import trigger_eval_job, display_evaluation_status, trigger_eval_sweep, eval_log_print, eval_stop
 from typing import Optional, Set, Tuple
 import re
 
-_ALLOWED_KEYS = {"task", "tp_size", "num_fewshot", "batch_size", "seq_length", "dp_size"}
+_ALLOWED_KEYS = {"task", "tp_size", "num_fewshot", "batch_size", "seq_length", "dp_size", "limit"}
 _TASK_NAME_RE = re.compile(r"^[A-Za-z0-9_.:\-\/]+$")
 
 def _validate_task_and_reason(task: str, allowed_tasks: Optional[Set[str]] = None) -> Tuple[bool, str]:
@@ -65,9 +65,9 @@ def _validate_task_and_reason(task: str, allowed_tasks: Optional[Set[str]] = Non
                     return False, f"invalid task name: {v}"
                 if allowed_tasks is not None and v not in allowed_tasks:
                     return False, f"task not allowed: {v}"
-            else:
-                if not re.fullmatch(r"\d+", v):
-                    return False, f"value for '{k}' must be a non-negative integer: {v}"
+            # else:
+            #     if not re.fullmatch(r"\d+", v):
+            #         return False, f"value for '{k}' must be a non-negative integer: {v}"
 
             seen[k] = v
 
@@ -485,6 +485,10 @@ def model(ctx,
         else:
             config_table.add_row("Wandb API Key", "❌ No set")
     
+    custom_code_base_string = get_custom_code_base_from_env()
+    if len(custom_code_base_string) > 0:
+        config_table.add_row("Custom Code Base", custom_code_base_string)
+    
     # Add parallelism config
     config_table.add_section()
     config_table.add_row("Tensor Parallel", str(tp))
@@ -524,7 +528,8 @@ def model(ctx,
     elif dataset_list:
         # For list mode, use the validated dataset paths
         dataset_list_for_api = valid_dataset_paths
-    
+
+
     trigger_eval_job(run_id, 
                      model_name, 
                      project_name, 
@@ -540,8 +545,8 @@ def model(ctx,
                      dataset_tasks,
                      model_args,
                      wandb_project,
-                     wandb_api_key
-                     )
+                     wandb_api_key,
+                     custom_code_base_string)
 
 @eval.command(name='status')
 @click.argument('eval_id', required=True, type=str)
@@ -568,5 +573,16 @@ def log(ctx,
     cluster = os.getenv("MLP_CLUSTER", "il2")
     eval_log_print(evalution_id, cluster)
 
+
+@eval.command()
+@click.argument('evalution_id', required=True, type=str)
+@click.pass_context
+def stop(ctx, 
+          evalution_id):
+    project = os.getenv("MLP_PROJECT", "llm")
+    cluster = os.getenv("MLP_CLUSTER", "il2")
+    eval_stop(evalution_id, cluster)
+
+
 def add_command(cli_group):
     cli_group.add_command(eval)
