@@ -84,6 +84,69 @@ def _format_and_print_logs(log_text):
         else:
             print(line)
 
+def _format_and_print_single_line(line):
+    """Format and print a single log line"""
+    import re
+    from datetime import datetime
+    
+    line = line.strip()
+    if not line:
+        return
+        
+    # Try to parse JSON logs first
+    if line.startswith('{"result":'):
+        try:
+            import json
+            log_entry = json.loads(line)
+            result = log_entry.get('result', {})
+            content = result.get('content', '')
+            
+            if content:
+                # Extract and format timestamp if present in content
+                timestamp_match = re.search(r'time="([^"]+)"', content)
+                if timestamp_match:
+                    timestamp_str = timestamp_match.group(1)
+                    try:
+                        dt = datetime.fromisoformat(timestamp_str.replace('Z', '+00:00'))
+                        formatted_time = dt.strftime('%H:%M:%S')
+                        content = re.sub(r'time="[^"]+" level=info msg="', f'[{formatted_time}] ', content)
+                        content = re.sub(r'" argo=true.*$', '', content)
+                    except:
+                        pass
+                
+                print(content)
+            return
+        except:
+            pass
+    
+    # Handle regular text logs
+    # Extract and format timestamp if present
+    timestamp_match = re.search(r'time="([^"]+)"', line)
+    if timestamp_match:
+        timestamp_str = timestamp_match.group(1)
+        try:
+            dt = datetime.fromisoformat(timestamp_str.replace('Z', '+00:00'))
+            formatted_time = dt.strftime('%H:%M:%S')
+            line = re.sub(r'time="[^"]+" level=info msg="', f'[{formatted_time}] ', line)
+            line = re.sub(r'" argo=true.*$', '', line)
+        except:
+            pass
+    
+    # Clean up common patterns and print
+    if line.startswith('==='):
+        print(f"\n{line}")
+    elif line.startswith('ERROR:') or line.startswith('WARNING:'):
+        print(f"⚠️  {line}")
+    elif line.startswith('Successfully installed') or line.startswith('Saved'):
+        print(f"✅ {line}")
+    elif 'MB/s eta' in line or '━━━━━' in line:
+        # Skip download progress bars
+        return
+    elif line.startswith('[notice]'):
+        print(f"ℹ️  {line}")
+    else:
+        print(line)
+
 MM_BASE_URL = "http://model-management-service.infra.svc.cluster.local:5000"
 
 
@@ -104,6 +167,7 @@ def trigger_eval_job(
     model_args: str = None,
     wandb_project: str = None,
     wandb_api_key: str = None,
+    custom_code_base_string: str = None
 ) -> bool:
     cluster = os.environ.get("MLP_CLUSTER", "il2")
     project = os.environ.get("MLP_PROJECT", "llm")
@@ -125,6 +189,9 @@ def trigger_eval_job(
         "eval_tokenizer_path": tokenizer_path,
         "status": "start",
         "validation_batch_size": validation_batch_size,
+        "code_info": {
+            "CUSTOM_CODE_BASE_STRING": custom_code_base_string.strip() if custom_code_base_string else ""
+        }
     }
     
     # Add dataset_tasks to data if provided
@@ -253,6 +320,18 @@ def trigger_eval_sweep(spec: dict, cluster, project, run_id=None):
         print("YAML file is empty or invalid.")
         return
 
+    models = config['generation']['models']
+    if len(models) == 0:
+        print("No models specified in the spec file.")
+        return
+
+    unique_model_names = set()
+    for model in models:
+        if 'model_name' in model and model['model_name'] in unique_model_names:
+            print(f"Duplicate model_name found in spec: {model['model_name']}. Please ensure all model names are unique.")
+            return
+        unique_model_names.add(model['model_name'])
+
     data = {
         "evaluation_id": run_id,
         "spec": config,
@@ -270,6 +349,7 @@ def trigger_eval_sweep(spec: dict, cluster, project, run_id=None):
             evaluation_id = response_data.get('evaluation_id')
             if evaluation_id:
                 print(f"Sweep workflow submmited. you can use the highlight command: \033[92manc eval log {evaluation_id}\033[0m to check the logs")
+                print(f"You can use the highlight command to stop it: \033[93manc eval stop {evaluation_id}\033[0m if needed")
             else:
                 print("Evaluation failed, didn't get the evaluation id")
         else:
@@ -286,8 +366,133 @@ def trigger_eval_sweep(spec: dict, cluster, project, run_id=None):
     except Exception as e:
         print(f"Sorry, your command run failed, you can try again or reach out infra team")
 
+def _resolve_argo_server_url(cluster_name):
+        if cluster_name == "il2":
+            return "https://10.218.61.160"
+        elif cluster_name == "hb":
+            return "https://10.53.139.209"
+        raise ValueError(f"Unsupported cluster: {cluster_name} for argo server url")
 
-def eval_log_print(evaluation_id, cluster):
+def eval_log_print(evaluation_id, cluster, follow=True):
+    import time
+    
+    namespace = "argo"
+
+    # First, get workflow_name from evaluation service
+    conn = Connection(url=MM_BASE_URL)
+    response = conn.get(f"/evaluations/{evaluation_id}")
+    
+    if response.status_code != 200:
+        print(f"Failed to get evaluation info: HTTP {response.status_code} {response.text}")
+        return
+    
+    try:
+        data = response.json()
+        workflow_name = data.get('train_job_url')
+        if not workflow_name or workflow_name.startswith("http"):
+            print(f"No workflow found for evaluation {evaluation_id}.")
+            return
+        
+        print(f"Found workflow: {workflow_name} for evaluation {evaluation_id}")
+        if follow:
+            print("Following logs (Ctrl+C to stop)...")
+    except Exception as e:
+        print(f"Failed to parse evaluation response: {e}")
+        return
+    
+    base_url = _resolve_argo_server_url(cluster)
+    logs_url = f"{base_url}/api/v1/workflows/{namespace}/{workflow_name}/log"
+    
+    # Stream logs with follow capability
+    last_log_content = ""
+    last_log_lines_count = 0
+    first_run = True
+    
+    try:
+        while True:
+            # Get workflow status to check if it's still running
+            wf_url = f"{base_url}/api/v1/workflows/{namespace}/{workflow_name}"
+            try:
+                wf_resp = requests.get(wf_url, verify=False)
+                if wf_resp.status_code == 200:
+                    wf = wf_resp.json()
+                    phase = wf.get('status', {}).get('phase', 'Unknown')
+                    
+                    # Get aggregated logs - try multiple parameter combinations
+                    param_variants = [
+                        {},  # No parameters
+                        {"logOptions.follow": "false"},
+                        {"follow": "false"},
+                        {"logOptions.container": "main"},
+                    ]
+                    
+                    log_found = False
+                    for params in param_variants:
+                        agg_resp = requests.get(logs_url, params=params, verify=False)
+                        
+                        if first_run:
+                            print(f"Debug: Trying logs with params {params}: HTTP {agg_resp.status_code}")
+                        
+                        if agg_resp.status_code == 200:
+                            current_log_content = agg_resp.text.strip()
+                            
+                            if current_log_content:
+                                log_found = True
+                                if current_log_content != last_log_content:
+                                    current_lines = current_log_content.split('\n')
+                                    
+                                    if first_run:
+                                        # First run: show all existing logs
+                                        print("==== Workflow Logs ====")
+                                        _format_and_print_logs(current_log_content)
+                                        first_run = False
+                                    else:
+                                        # Subsequent runs: only print new lines
+                                        new_lines = current_lines[last_log_lines_count:]
+                                        
+                                        if new_lines and (new_lines[0] or len(new_lines) > 1):  # Skip if only empty line
+                                            for line in new_lines:
+                                                if line.strip():  # Only print non-empty lines
+                                                    _format_and_print_single_line(line)
+                                    
+                                    last_log_content = current_log_content
+                                    last_log_lines_count = len(current_lines)
+                                break
+                    
+                    if first_run and not log_found:
+                        print(f"No logs found with any parameter combination. Workflow phase: {phase}")
+                        first_run = False
+                    
+                    # Check if workflow is completed
+                    if not follow or phase in ['Succeeded', 'Failed', 'Error']:
+                        if phase in ['Succeeded']:
+                            print(f"\n✅ Workflow {workflow_name} completed successfully!")
+                        elif phase in ['Failed', 'Error']:
+                            print(f"\n❌ Workflow {workflow_name} failed with status: {phase}")
+                        break
+                        
+                else:
+                    print(f"Failed to get workflow status: HTTP {wf_resp.status_code}")
+                    if not follow:
+                        break
+                        
+            except Exception as e:
+                print(f"Error fetching logs: {e}")
+                if not follow:
+                    break
+            
+            if follow and phase not in ['Succeeded', 'Failed', 'Error']:
+                time.sleep(2)  # Wait 2 seconds before next poll
+            else:
+                break
+                
+    except KeyboardInterrupt:
+        print(f"\n🛑 Log following stopped by user")
+    except Exception as e:
+        print(f"\n❌ Error in log streaming: {e}")
+    
+
+def eval_stop(evaluation_id: str, cluster):
     namespace = "argo"
 
     # First, get workflow_name from evaluation service
@@ -309,140 +514,41 @@ def eval_log_print(evaluation_id, cluster):
     except Exception as e:
         print(f"Failed to parse evaluation response: {e}")
         return
-
-    def _resolve_argo_server_url(cluster_name):
-        if cluster_name == "il2":
-            return "https://10.218.61.160"
-        elif cluster_name == "hb":
-            return "https://10.53.139.209"
-
-        raise ValueError(f"Unsupported cluster: {cluster_name} for argo server url")
     
     base_url = _resolve_argo_server_url(cluster)
-    print(f"Debug: Using Argo server URL: {base_url}")
-
-    # 1) Get workflow to discover nodes/pods
+    
+    # First check workflow status
     wf_url = f"{base_url}/api/v1/workflows/{namespace}/{workflow_name}"
     try:
         wf_resp = requests.get(wf_url, verify=False)
-    except Exception as e:
-        print(f"Failed to connect to Argo server: {e}")
-        return
-
-    if wf_resp.status_code != 200:
-        print(f"Failed to get workflow: HTTP {wf_resp.status_code} {wf_resp.text}")
-        return
-
-    try:
-        wf = wf_resp.json()
-    except Exception as e:
-        print(f"Failed to parse workflow JSON: {e}")
-        return
-
-    status = wf.get('status', {})
-    nodes = status.get('nodes', {}) or {}
-    print(f"Debug: Found {len(nodes)} nodes in workflow status")
-    
-    # Check if nodes are offloaded
-    if status.get('offloadNodeStatusVersion'):
-        print(f"Debug: Nodes are offloaded (version: {status.get('offloadNodeStatusVersion')})")
-        # Try to get nodes from separate endpoint
-        nodes_url = f"{base_url}/api/v1/workflows/{namespace}/{workflow_name}/nodes"
-        try:
-            nodes_resp = requests.get(nodes_url, verify=False)
-            if nodes_resp.status_code == 200:
-                nodes_data = nodes_resp.json()
-                nodes = nodes_data.get('items', [])
-                print(f"Debug: Retrieved {len(nodes)} nodes from nodes endpoint")
-                # Convert list to dict format
-                if isinstance(nodes, list):
-                    nodes_dict = {}
-                    for node in nodes:
-                        node_id = node.get('id', '')
-                        if node_id:
-                            nodes_dict[node_id] = node
-                    nodes = nodes_dict
-        except Exception as e:
-            print(f"Debug: Failed to get nodes from separate endpoint: {e}")
-
-    logs_url = f"{base_url}/api/v1/workflows/{namespace}/{workflow_name}/log"
-
-    # 2) Try aggregated workflow logs first (covers all steps/pods)
-    print("Debug: Trying aggregated logs...")
-    try:
-        # Try different parameter combinations
-        param_variants = [
-            {},  # No parameters
-            {"logOptions.follow": "false"},
-            {"follow": "false"},
-            {"logOptions.container": "main"},
-        ]
-        
-        for params in param_variants:
-            agg_resp = requests.get(logs_url, params=params, verify=False)
-            print(f"Debug: Aggregated logs attempt with params {params}: HTTP {agg_resp.status_code}")
-            if agg_resp.status_code == 200 and agg_resp.text.strip():
-                print("==== Aggregated Workflow Logs ====")
-                _format_and_print_logs(agg_resp.text)
+        if wf_resp.status_code == 200:
+            wf = wf_resp.json()
+            phase = wf.get('status', {}).get('phase', 'Unknown')
+            print(f"Current workflow status: {phase}")
+            
+            if phase in ['Succeeded', 'Failed', 'Error']:
+                print(f"⚠️  Workflow is already completed with status: {phase}")
                 return
-    except Exception as e:
-        print(f"Debug: Aggregated logs failed: {e}")
-
-    # 3) Try per-node logs using nodeId
-    node_items = []
-    for node_id, node in nodes.items():
-        node_items.append({
-            "id": node_id,
-            "displayName": node.get("displayName") or node.get("name") or node_id,
-            "type": node.get("type"),
-            "finishedAt": node.get("finishedAt") or node.get("startedAt") or "",
-            "podName": node.get("podName")
-        })
-    
-    print(f"Debug: Processing {len(node_items)} nodes for individual logs")
-    # sort by finishedAt for deterministic order
-    node_items.sort(key=lambda x: x.get("finishedAt") or "")
-
-    printed_any = False
-    for item in node_items:
-        print(f"Debug: Trying logs for node {item['displayName']} ({item['type']})")
-        
-        # Try different parameter combinations for individual nodes
-        params_variants = [
-            {"nodeId": item["id"]},
-            {"nodeId": item["id"], "logOptions.follow": "false"},
-            {"nodeId": item["id"], "follow": "false"},
-            {"nodeId": item["id"], "logOptions.container": "main"},
-        ]
-        
-        if item.get("podName"):
-            params_variants.extend([
-                {"podName": item["podName"]},
-                {"podName": item["podName"], "logOptions.container": "main"},
-                {"podName": item["podName"], "container": "main"},
-            ])
-
-        log_text = None
-        last_err = None
-        for params in params_variants:
-            try:
-                resp = requests.get(logs_url, params=params, verify=False)
-                if resp.status_code == 200 and resp.text.strip():
-                    log_text = resp.text
-                    break
-                else:
-                    last_err = f"HTTP {resp.status_code}: {resp.text[:200]}"
-            except Exception as e:
-                last_err = str(e)
-
-        if log_text:
-            printed_any = True
-            print("")
-            print(f"==== Step: {item['displayName']} ({item['type']}) ====")
-            _format_and_print_logs(log_text)
         else:
-            print(f"Debug: No logs found for {item['displayName']}: {last_err}")
-
-    if not printed_any:
-        print("No logs found for this workflow.")
-    
\ No newline at end of file
+            print(f"Failed to get workflow status: HTTP {wf_resp.status_code}")
+            return
+    except Exception as e:
+        print(f"Error getting workflow status: {e}")
+        return
+    
+    # Stop the workflow using Argo API
+    stop_url = f"{base_url}/api/v1/workflows/{namespace}/{workflow_name}/stop"
+    
+    try:
+        print(f"Stopping workflow {workflow_name}...")
+        stop_resp = requests.put(stop_url, verify=False)
+        
+        if stop_resp.status_code == 200:
+            print(f"Successfully stop for workflow {workflow_name}")
+            
+        else:
+            print(f"❌ Failed to stop workflow: HTTP {stop_resp.status_code}")
+            print(f"Response: {stop_resp.text}")
+            
+    except Exception as e:
+        print(f"❌ Error stopping workflow: {e}")
