@@ -142,3 +142,34 @@ class ConfigManager:
         project = os.environ.get(self.ENV_PROJECT) or cluster_info.get('project', '')
 
         return project, cluster, personal
+
+def validate_path_exists(path: str) -> bool:
+    """Validate if the path exists"""
+    if not os.path.exists(path):
+        print(f"Your path {path} is invalid, path not exists")
+        return False
+    return True
+
+
+def get_custom_code_base_from_env() -> str:
+    custom_code_base_string = ""
+    # Read code base paths from environment variables
+    harness_code_base = os.getenv("HARNESS_CODE_BASE", "")
+    ocean_code_base = os.getenv("OCEAN_CODE_BASE", "")
+    nemo_code_base = os.getenv("NEMO_CODE_BASE", "")
+    megatron_code_base = os.getenv("MEGATRON_CODE_BASE", "")
+    
+    path_list = [path for path in [harness_code_base, ocean_code_base, nemo_code_base, megatron_code_base] if path != ""]
+    
+    for path in path_list:
+        if not validate_path_exists(path):
+            print(f"Your path {path} is invalid, path not exists")
+            sys.exit(1)
+   
+    custom_code_base_string += f"OCEAN={ocean_code_base}," if ocean_code_base else ""
+    custom_code_base_string += f"NEMO={nemo_code_base}," if nemo_code_base else ""
+    custom_code_base_string += f"MEGATRON={megatron_code_base}," if megatron_code_base else ""
+    custom_code_base_string += f"LM_EVALUATION_HARNESS={harness_code_base}," if harness_code_base else ""
+    if custom_code_base_string.endswith(","):
+        custom_code_base_string = custom_code_base_string[:-1]
+    return custom_code_base_string
\ No newline at end of file
