@@ -43,6 +43,7 @@ class AncComposerBuffer:
         split_fn=None,
         freeze_batch_interval=None,
         enable_logging=True,
+        # get_from_split_chunks_prob=0.6,
     ):
         self.buffer = []
         self.total_token_length = 0
@@ -59,6 +60,8 @@ class AncComposerBuffer:
         self.freeze_splits_buffer = []
         self.unfrozen_splits_buffer = []
         self.enable_logging = enable_logging
+        # self.random_state = None
+        # self.get_from_split_chunks_prob = get_from_split_chunks_prob
 
     def put(self, item):
         """Add an item to the buffer and return total token length.
@@ -124,6 +127,10 @@ class AncComposerBuffer:
         """
         if len(self.unfrozen_splits_buffer) == 0:
             return
+        # if self.random_state is None:
+        #     self.random_state = random.Random(self.batch_count)
+        # if self.random_state.random() > self.get_from_split_chunks_prob:
+        #     return
         split_chunk = self.unfrozen_splits_buffer.pop(0)
         if split_chunk[0] <= target_length:
             return split_chunk
@@ -137,7 +144,7 @@ class AncComposerBuffer:
             to_return, remain = self.split_fn(split_chunk[1], target_length)
             remain_length = self._get_token_length(remain)
             assert self._get_token_length(to_return) == target_length
-            self.freeze_splits_buffer.append((remain_length, remain))
+            self.freeze_splits_buffer.append((remain_length, remain, self.batch_count))
             return (target_length, to_return)
     
     def get_with_split(self, target_length, ignore_split_freezing=False):
@@ -181,7 +188,7 @@ class AncComposerBuffer:
             )
             self.cur_idx = max_length_idx
         else:
-            self.freeze_splits_buffer.append((remain_length, remain))
+            self.freeze_splits_buffer.append((remain_length, remain, self.batch_count))
             self.total_token_length -= max_length
             self.delete_idx.add(max_length_idx)
         return (target_length, to_return)
@@ -204,8 +211,9 @@ class AncComposerBuffer:
         
         Adds split chunks from freeze_splits_buffer to unfrozen_splits_buffer.
         """
-        self.unfrozen_splits_buffer += self.freeze_splits_buffer
-        self.freeze_splits_buffer = []
+        to_freeze = [(i[0], i[1]) for i in self.freeze_splits_buffer if self.batch_count - i[2] > self.freeze_batch_interval]
+        self.unfrozen_splits_buffer += to_freeze
+        self.freeze_splits_buffer = [i for i in self.freeze_splits_buffer if self.batch_count - i[2] <= self.freeze_batch_interval]
         if self.enable_logging:
             print(
                 f"[Composer] item count in unfrozen_splits_buffer: "
@@ -225,12 +233,14 @@ class AncComposerBuffer:
         buffer = [self.buffer[i] for i in range(len(self.buffer)) if i not in self.delete_idx]
         self.buffer = buffer
         self.delete_idx = set()
+        #print(f"=====self.total_token_length: {self.total_token_length}, {[i[0] for i in self.unfrozen_splits_buffer]}, {[(i[0], i[2]) for i in self.freeze_splits_buffer]}, {self.batch_count}, {self.freeze_batch_interval}")
         assert sum([i[0] for i in self.buffer]) == self.total_token_length
         if self.enable_split_freeze:
             self.batch_count += 1
-            if self.batch_count == self.freeze_batch_interval:
-                self.batch_count = 0
-                self.unfrozen_splits()
+            # if self.batch_count == self.freeze_batch_interval:
+            #     self.batch_count = 0
+            self.unfrozen_splits()
+        #print(f"=====self.total_token_length: {self.total_token_length}, {[i[0] for i in self.unfrozen_splits_buffer]}, {[(i[0], i[2]) for i in self.freeze_splits_buffer]}, {self.batch_count}, {self.freeze_batch_interval}")
 
     def combine_buffers(self):
         """Combine buffers.
@@ -238,7 +248,7 @@ class AncComposerBuffer:
         Adds split chunks from unfrozen_splits_buffer and freeze_splits_buffer to the main buffer.
         It is usually called when running to the end of the dataset
         """
-        self.buffer += self.unfrozen_splits_buffer + self.freeze_splits_buffer
+        self.buffer += self.unfrozen_splits_buffer + [(i[0], i[1]) for i in self.freeze_splits_buffer]
         self.unfrozen_splits_buffer = []
         self.freeze_splits_buffer = []
         self.total_token_length = sum([i[0] for i in self.buffer])
@@ -252,6 +262,7 @@ class AncComposerBuffer:
         states['delete_idx'] = self.delete_idx
         states['cur_idx'] = self.cur_idx
         states['batch_count'] = self.batch_count
+        # states['random_state'] = self.random_state
         return states
 
     def _set_ckpt(self, state):
@@ -262,7 +273,7 @@ class AncComposerBuffer:
         self.delete_idx = state['delete_idx']
         self.cur_idx = state['cur_idx']
         self.batch_count = state['batch_count']
-
+        # self.random_state = state['random_state']
 
 class AncComposer:
     """Main composer class that handles sequence composition and batch creation.
@@ -478,4 +489,4 @@ if __name__ == "__main__":
                 batch_count += 1
                 token_lengths = []
         if batch_count >= batch_limit:
-            break
\ No newline at end of file
+            break
