@@ -8,6 +8,7 @@ from anc.data.utils import DataNotReady
 import logging
 import datetime
 import multiprocessing as mp
+from .utils import line_print
 
 
 def fake_collate(x):
@@ -79,7 +80,7 @@ class AncDataLoader:
         persistent_workers=False,
         use_spawn=False,
         use_fake_data=False,
-        compose_buffer_ratio=2.0,
+        compose_buffer_ratio=20.0,
     ):
     # TODO: add support to get rank and world from torch env
         self.paths = paths
@@ -118,7 +119,7 @@ class AncDataLoader:
             self.sampler = AncMultiSourceSampler(
                 dataset=self.dataset,
                 ratios=ds_ratios,
-                batch_size=batch_size,
+                batch_size=1 if enable_compose else batch_size,
                 world=world,
                 rank=rank,
                 num_workers=num_workers,
@@ -129,6 +130,7 @@ class AncDataLoader:
                 global_shuffle=global_shuffle,
                 chunk_granularity=chunk_granularity,
                 mem_save_mode=sampler_use_mem_save_mode,
+                is_train=is_train,
             )
         else:
             self.sampler = AncSampler(
@@ -144,6 +146,7 @@ class AncDataLoader:
                 global_shuffle=global_shuffle,
                 chunk_granularity=chunk_granularity,
                 mem_save_mode=sampler_use_mem_save_mode,
+                is_train=is_train,
             )
         self.dataset.set_sampler(self.sampler)
         if data_borrow:
@@ -283,7 +286,7 @@ class AncDataLoader:
         need_logging = self.logging and need_logging
         # if getting data is too slow, log the time
         if need_logging or consumed_time > self.logging_threshold:
-            logging.info(f"__next__ called time {now} pid {os.getpid()} rank {self.rank} return step {self.step} consumed time {consumed_time} data with buffer {self.loader_iter._data_queue.qsize()}")
+            line_print(f"__next__ called time {now} pid {os.getpid()} rank {self.rank} return step {self.step} consumed time {consumed_time} data with buffer {self.loader_iter._data_queue.qsize()}")
         if self.use_fake_data and self.first_batch is None:
             self.first_batch = data
         return data
