@@ -11,6 +11,7 @@ import pickle
 import itertools
 import logging
 import random
+import copy
 
 class AncDataset(IterableDataset):
     r'''
@@ -47,9 +48,22 @@ class AncDataset(IterableDataset):
         if dataset_type == "parquet":
             # TODO: add decode_fn for parquet
             if isinstance(filepaths[0], list):
-                self.ds = [ParquetConcateDataset(filepaths[i], columns=ds_args.get("columns")) for i in range(len(filepaths))]
+                self.ds = [
+                    ParquetConcateDataset(
+                        filepaths[i],
+                        columns=ds_args.get("columns"),
+                        metadata=ds_args.get("metadata")[i] if ds_args.get("metadata") is not None else None,
+                    )
+                    for i in range(len(filepaths))
+                ]
             else:
-                self.ds = [ParquetConcateDataset(filepaths, columns=ds_args.get("columns"))]
+                self.ds = [
+                    ParquetConcateDataset(
+                        filepaths,
+                        columns=ds_args.get("columns"),
+                        metadata=ds_args.get("metadata"),
+                    )
+                ]
         elif dataset_type == "jsonl":
             assert isinstance(filepaths[0], str), f"jsonl dataset does not support multisources {filepaths}"
             assert len(filepaths) == 1, f"jsonl dataset only support single file, but got {len(filepaths)}. Please convert jsonl dataset to parquet first"
@@ -99,7 +113,8 @@ class AncDataset(IterableDataset):
             self._set_ckpt(self.ds_states[wid])
         logging.debug(f"iter information: len(sampler) - {len(self.sampler)}, worker id - {wid}, num_workers - {num_workers}")
         # use islice to get the iterator of sampler only for current worker
-        sampler_iter = itertools.islice(iter(self.sampler), wid, len(self.sampler), num_workers)
+        # sampler_iter = itertools.islice(iter(self.sampler), wid, len(self.sampler), num_workers)
+        sampler_iter = iter(self.sampler)
         yield from self.get_generator(sampler_iter)
 
     def get_generator(self, sampler_iter):
@@ -193,7 +208,7 @@ class AncDataset(IterableDataset):
             state['composer'] = self.composer._get_ckpt()
         state['cur_step'] = self.cur_step
         state['remain_data'] = self.raw_data
-        return state
+        return copy.deepcopy(state)
 
     def _set_ckpt(self, state):
         if 'sampler' in state and self.sampler is not None:
