@@ -1,9 +1,13 @@
 from torch.utils.data.sampler import *
 from torch.utils.data.distributed import DistributedSampler
 import random
+import torch
 import time
 import math
 import copy
+import bisect
+from .utils import line_print
+
 
 class CustomizedRange:
     def __init__(self, start, end, seed=0):
@@ -42,30 +46,47 @@ class CustomizedRange:
 class RangesHolder:
     def __init__(self, list_of_ranges):
         self.list_of_ranges = list_of_ranges
+        self.accu_lengths = self.create_accu_length(list_of_ranges)
         self.length = sum(len(r) for r in list_of_ranges)
         self.expanded = {}
+        self.i = 0
     
     def __len__(self):
         return self.length
 
+    def create_accu_length(self, list_of_ranges):
+        accu_length = 0
+        accu_lengths = []
+        for rng in list_of_ranges:
+            accu_lengths.append(accu_length)
+            accu_length += len(rng)
+        accu_lengths.append(accu_length)
+        return accu_lengths
+
+    def _get_start_idx(self, start):
+        idx = bisect.bisect_right(self.accu_lengths, start) - 1
+        assert 0 <= idx < len(self.accu_lengths)
+        return idx
+
     def get_start_and_end_ranges(self, start, length):
         start_range_id, start_range_offset = 0, 0
-        accu_length = 0
+        n = len(self.list_of_ranges)
         end = start + length
-        end_range_id = len(self.list_of_ranges) - 1
+        end_range_id = n - 1
         end_range_offset = len(self.list_of_ranges[end_range_id])
-        for i in range(len(self.list_of_ranges)):
-            if start < accu_length + len(self.list_of_ranges[i]):
-                start_range_id = i
-                start_range_offset = start - accu_length
-                break
-            accu_length += len(self.list_of_ranges[i])
-        for i in range(start_range_id, len(self.list_of_ranges)):
+        if self.accu_lengths[self.i] <= start < self.accu_lengths[self.i + 1]:
+            start_range_id = self.i
+        else:
+            start_range_id = self._get_start_idx(start)
+        start_range_offset = start - self.accu_lengths[start_range_id]
+        accu_length = self.accu_lengths[start_range_id]
+        for i in range(start_range_id, n):
             accu_length += len(self.list_of_ranges[i])
             if end < accu_length:
                 end_range_id = i
                 end_range_offset = end - (accu_length - len(self.list_of_ranges[i]))
                 break
+        self.i = end_range_id
         return start_range_id, start_range_offset, end_range_id, end_range_offset
 
     def slice(self, start, length):
@@ -123,6 +144,7 @@ class RangesHolder:
     def random_reset(self, idx):
         self.expanded = {}
         self.list_of_ranges = self.list_of_ranges[idx:] + self.list_of_ranges[:idx]
+        self.accu_lengths = self.create_accu_length(self.list_of_ranges)
 
     
 
@@ -169,6 +191,8 @@ class AncSampler(DistributedSampler):
         ratios = [1],
         chunk_granularity = -1,
         mem_save_mode = False,
+        ds_id=0,
+        is_train=True,
     ):
         if hasattr(dataset, 'get_sub_lengths'):
             self.sub_lengths = dataset.get_sub_lengths()
@@ -191,6 +215,8 @@ class AncSampler(DistributedSampler):
         self.drop_last = drop_last
         self.chunk_granularity = chunk_granularity
         self.mem_save_mode = mem_save_mode
+        self.ds_id = ds_id
+        self.is_train = is_train
         if self.mem_save_mode:
             assert self.chunk_granularity == -1, "chunk_granularity is not supported when mem_save_mode is True"
         if len(ratios) == 1:
@@ -212,6 +238,7 @@ class AncSampler(DistributedSampler):
         self.shuffle = shuffle
         if getattr(self, "dataset", None) is not None:
             self.dataset = None
+        self.random = random.Random()
 
     def _get_indices_with_ratio(self, indices, ratio):
         ratio_int = math.floor(ratio)
@@ -224,7 +251,7 @@ class AncSampler(DistributedSampler):
             else:
                 tmp_indices = indices[:]
                 if self.shuffle:
-                    random.shuffle(tmp_indices)
+                    self.random.shuffle(tmp_indices)
                 if self.chunk_granularity > 0:
                     num_chunks = (len(tmp_indices) + self.chunk_granularity - 1) // self.chunk_granularity
                     for i in range(num_chunks):
@@ -235,12 +262,12 @@ class AncSampler(DistributedSampler):
             if isinstance(indices, CustomizedRange):
                 tmp_indices = indices.clone(ratio_int)
                 length = math.ceil(ratio_float * len(tmp_indices))
-                start = 0 if not self.shuffle else random.randint(0, len(tmp_indices) - length)
+                start = 0 if not self.shuffle else self.random.randint(0, len(tmp_indices) - length)
                 res.append(tmp_indices.slice(start, length))
             else:
                 tmp_indices = indices[:]
                 if self.shuffle:
-                    random.shuffle(tmp_indices)
+                    self.random.shuffle(tmp_indices)
                 tmp_indices = tmp_indices[:math.ceil(ratio_float * len(tmp_indices))]
                 if self.chunk_granularity > 0:
                     num_chunks = (len(tmp_indices) + self.chunk_granularity - 1) // self.chunk_granularity
@@ -249,14 +276,14 @@ class AncSampler(DistributedSampler):
                 else:
                     res.append(tmp_indices)
         if self.shuffle:
-            random.shuffle(res)
+            self.random.shuffle(res)
         return res
 
     def _get_indices(self, sub_lengths, offset=0, return_sub_indices=False, ratio=[1]):
         if self.mem_save_mode:
             assert return_sub_indices is True
         indices_list = []
-        random.seed(self.seed)
+        self.random.seed(self.seed)
         accumulate_length = offset
         if len(ratio) == 1:
             ratio = ratio * len(sub_lengths)
@@ -272,7 +299,7 @@ class AncSampler(DistributedSampler):
             accumulate_length += item
 
         if self.shuffle:
-            random.shuffle(indices_list)
+            self.random.shuffle(indices_list)
         if return_sub_indices:
             return indices_list
         indices = []
@@ -281,6 +308,12 @@ class AncSampler(DistributedSampler):
         return indices
 
     def create_chunk_indices_from_bz_and_worker(self, indices):
+        worker_info = torch.utils.data.get_worker_info()
+        #if os.getenv("WID_FOR_DEBUG") is not None:
+        #    actual_wid = int(os.getenv("WID_FOR_DEBUG"))
+        #else:
+        #    actual_wid = worker_info.id if worker_info is not None else 0
+        actual_wid = worker_info.id if worker_info is not None else 0
         # calculate num batches per worker
         num_batches = (self.num_samples + self.batch_size - 1) // self.batch_size
         last_batch_size = self.num_samples - (num_batches - 1) * self.batch_size
@@ -303,33 +336,48 @@ class AncSampler(DistributedSampler):
             start_idx_per_worker[i] = start_idx_per_worker[i - 1] + num_samples_per_worker[i - 1]
 
         if self.repeat and self.shuffle:
-            random.seed(self.seed)
+            self.random.seed(self.seed)
+        inner_epoch_count = 1
         while True:
             self.inner_epoch_count += 1
-            current_idx_per_worker = [i for i in start_idx_per_worker]
-            for i in range(num_batches):
-                wid = i % self.num_workers
-                start_idx = current_idx_per_worker[wid]
-                end_idx = start_idx + self.batch_size
-                is_last_batch = (end_idx - start_idx_per_worker[wid]) >= num_samples_per_worker[wid] and not self.repeat
-                end_idx = min(end_idx, start_idx_per_worker[wid] + num_samples_per_worker[wid])
-                if self.mem_save_mode:
-                    yield indices.get_value_from(start_idx, end_idx, self.shuffle), is_last_batch
-                else:
-                    yield indices[start_idx: end_idx], is_last_batch
-                current_idx_per_worker[wid] = end_idx
+            # current_idx_per_worker = [i for i in start_idx_per_worker]
+            # for i in range(num_batches):
+            #     wid = i % self.num_workers
+            #     start_idx = current_idx_per_worker[wid]
+            #     end_idx = start_idx + self.batch_size
+            #     is_last_batch = (end_idx - start_idx_per_worker[wid]) >= num_samples_per_worker[wid] and not self.repeat
+            #     end_idx = min(end_idx, start_idx_per_worker[wid] + num_samples_per_worker[wid])
+            #     if self.mem_save_mode:
+            #         yield indices.get_value_from(start_idx, end_idx, self.shuffle), is_last_batch
+            #     else:
+            #         yield indices[start_idx: end_idx], is_last_batch
+            #     current_idx_per_worker[wid] = end_idx
 
+            _start_idx = start_idx_per_worker[actual_wid]
+            for i in range(num_batches_per_each_worker[actual_wid]):
+                _end_idx = _start_idx + self.batch_size
+                is_last_batch = (_end_idx - start_idx_per_worker[actual_wid]) >= num_samples_per_worker[actual_wid] and not self.repeat
+                _end_idx = min(_end_idx, start_idx_per_worker[actual_wid] + num_samples_per_worker[actual_wid])
+                if self.mem_save_mode:
+                    yield indices.get_value_from(_start_idx, _end_idx, self.shuffle), is_last_batch
+                else:
+                    yield indices[_start_idx: _end_idx], is_last_batch
+                _start_idx = _end_idx
+
+            if self.is_train:
+                line_print(f"dataset {self.ds_id} on rank {self.rank} wid {actual_wid} get into {inner_epoch_count + 1} epoch")
+            inner_epoch_count += 1
             if not self.repeat:
                 break
             if self.global_shuffle:
                 assert self.mem_save_mode is False
-                random.shuffle(indices)
+                self.random.shuffle(indices)
             elif self.shuffle:
                 if self.mem_save_mode:
-                    shift_offset = random.randint(0, len(indices.list_of_ranges))
+                    shift_offset = self.random.randint(0, len(indices.list_of_ranges))
                     indices.random_reset(shift_offset)
                 else:
-                    shift_offset = random.randint(0, len(indices))
+                    shift_offset = self.random.randint(0, len(indices))
                     indices = indices[shift_offset:] + indices[:shift_offset]
     
     def __iter__(self):
@@ -346,7 +394,7 @@ class AncSampler(DistributedSampler):
                     accumulate_length += sum(item)
                     sub_indices_list += sub_indices
                 if self.shuffle:
-                    random.shuffle(sub_indices_list)
+                    self.random.shuffle(sub_indices_list)
                 if not self.mem_save_mode:
                     for item in sub_indices_list:
                         indices += item
@@ -355,7 +403,7 @@ class AncSampler(DistributedSampler):
             else:
                 indices = self._get_indices(self.sub_lengths, ratio=self.ratios)
                 if self.global_shuffle:
-                    random.shuffle(indices)
+                    self.random.shuffle(indices)
             
             if not self.drop_last:
                 # add extra samples to make it evenly divisible
@@ -393,12 +441,12 @@ class AncSampler(DistributedSampler):
         for i in range(self.resume_step):
             _ = next(indices_generator)
 
-        # yield empty indices to let workers follow the original order
-        # otherwise, the first generated indices would be consumed by worker 0
-        # while they should be consumed by cur_wid
-        cur_wid = self.resume_step % self.num_workers
-        for i in range(cur_wid):
-            yield [], False
+        # # yield empty indices to let workers follow the original order
+        # # otherwise, the first generated indices would be consumed by worker 0
+        # # while they should be consumed by cur_wid
+        # cur_wid = self.resume_step % self.num_workers
+        # for i in range(cur_wid):
+        #     yield [], False
 
         while True:
             try:
@@ -450,6 +498,7 @@ class AncMultiSourceSampler():
         global_shuffle = False,
         chunk_granularity = -1,
         mem_save_mode = False,
+        is_train = True,
     ):
         self.ratios = ratios
         self.seed = seed + rank
@@ -469,16 +518,23 @@ class AncMultiSourceSampler():
             global_shuffle=global_shuffle,
             chunk_granularity=chunk_granularity,
             mem_save_mode=mem_save_mode,
+            ds_id=i,
+            is_train=is_train,
         ) for i in range(len(dataset))]
         self.random = random.Random(self.seed)
         self.shuffle = shuffle
+        self.resume_from_ckpt = False
 
     def __iter__(self):
         iters = [iter(sampler) for sampler in self.samplers]
         iter_list = list(range(len(iters)))
-        if not self.shuffle:
+        worker_info = torch.utils.data.get_worker_info()
+        wid = worker_info.id if worker_info is not None else 0
+        if self.resume_from_ckpt:
+            self.resume_from_ckpt = False
+        elif True or not self.shuffle:
             # if shuffle is False, we want every epoch sampler output the same indices
-            self.random = random.Random(self.seed)
+            self.random = random.Random(self.seed + (wid + 123) * 100)
         while True:
             cur_iter_idx = self.random.choices(iter_list, weights=self.ratios)[0]
             cur_iter = iters[cur_iter_idx]
@@ -499,4 +555,5 @@ class AncMultiSourceSampler():
         assert len(state['samplers']) == len(self.samplers)
         for i, sampler in enumerate(self.samplers):
             sampler._set_ckpt(state['samplers'][i])
-        self.random.setstate(state['random_state'])
\ No newline at end of file
+        self.random.setstate(state['random_state'])
+        self.resume_from_ckpt = True
