@@ -140,6 +140,11 @@ class PretrainProcessor(Processor):
         else:
             self.data_to_skip = set()
 
+        try:
+            self.too_long_sample_threshold = int(os.getenv("ANC_TOO_LONG_THRESHOLD"))
+        except:
+            self.too_long_sample_threshold = None
+
     def transform(self, item, is_last_sample=False):
         """Transform a text item into tokenized sequences with loss masks.
         
@@ -154,16 +159,21 @@ class PretrainProcessor(Processor):
             logging.warning(f"skipping sample {item['filepath']}:{item['row_idx']} because it is in the skip list")
             return None
         tokens = self.tokenizer.txt2vec(item['content'])
+        if self.too_long_sample_threshold is not None and len(tokens) > self.too_long_sample_threshold:
+            logging.warning(f"Too long sample threshold is set, and sample length {len(tokens)} longer than threshold {self.too_long_sample_threshold}. Will drop")
+            return None
         if len(tokens) > self.max_seq_len and not self.seq_split_config.allow_split:
-            logging.warning(f"pretrain sample encounter with token length larger than {self.max_seq_len}, will ignore and continue")
+            logging.warning(f"pretrain sample encounter with token length {len(tokens)} larger than {self.max_seq_len}, will ignore and continue")
             return None
         loss_mask = [1] * len(tokens)
         # set mask between two special tokens <MASK:Z0k7RxyNaZ> and </MASK:Z0k7RxyNaZ> to zero
         # may need more complex logic when we have more special tokens, or mask logic becomes more complex
         set_to_zero = False
         successive_zero_count = 0
+        special_token_idxs = []
         for i in range(len(tokens)):
             if tokens[i] in self.special_token_ids:
+                special_token_idxs.append(i)
                 set_to_zero = not set_to_zero
                 loss_mask[i] = 0
                 successive_zero_count += 1
@@ -180,6 +190,11 @@ class PretrainProcessor(Processor):
                     return None
             else:
                 successive_zero_count = 0
+        # check if the first special token is the mask end token
+        if special_token_idxs and tokens[special_token_idxs[0]].startswith("</"):
+            # flip the loss mask
+            special_token_idxs = set(special_token_idxs)
+            loss_mask = [1 - item for i, item in enumerate(loss_mask) if i not in special_token_idxs]
         if sum(loss_mask) == 0:
             self.all_masked_sample_cnt += 1
             if self.sample_cnt >= 1000:
